#ifndef __Q_SOCK2_H__
#define __Q_SOCK2_H__

#include "qstdinc.h"

/* Address class */
class QNetAddr
{
public:
	/* Constructors */
	QNetAddr();												// Assign later
	QNetAddr(const QNetAddr&);				// Assign later
	QNetAddr(const char*);						// Use address:port
	QNetAddr(const char*, uint16_t);		// Use address, port
	QNetAddr(uint32_t, uint16_t);				// Use address, port
#ifndef _WIN32
	QNetAddr(int, bool bPeer = false);		// Use bound socket
#else
	QNetAddr(SOCKET, bool bPeer = false); // Use bound socket
#endif
	virtual ~QNetAddr();

	QNetAddr& operator=(const QNetAddr&);

	/* Same as constructors */
	virtual bool SetAddr(const char*);				// Use address:port
	virtual bool SetAddr(const char*, uint16_t);	// Use address,port
	virtual bool SetAddr(uint32_t, uint16_t);		// Use address,port
#ifndef _WIN32
	virtual bool SetAddr(int, bool bPeer = false);	// Use bound socket, true - local, false - remote
#else
	virtual bool SetAddr(SOCKET, bool bPeer = false);	// Use bound socket, true - local, false - remote
#endif

	/* Is address valid? */
	virtual bool IsValid();

	/* Access members */
	virtual uint32_t GetAddr();
	virtual uint16_t GetPort();
	virtual const char* GetAddrFull();				// In form adress:port
	virtual const char* GetAddrIP();				// IP address only

	/* Make copy */
	virtual QNetAddr* Duplicate();
	virtual void CopyTo(QNetAddr*);
	virtual void CopyFrom(const QNetAddr&);

protected:
	/* Reset all info */
	virtual void Reset();

	/* Set string variants */
	virtual void Expand();

	bool		m_bValid;
	uint32_t	m_nAddr;					// IP address
	uint16_t	m_nPort;					// Port value
	std::string m_sFullAddr;				// Full address
	std::string m_sIPOnly;					// IP only address
};
typedef std::vector<QNetAddr> QNetAddrs;


class QTCPSock
{
public:
	QTCPSock(QNetAddr*);
#ifndef _WIN32
	QTCPSock(int);
#else
	QTCPSock(SOCKET);
#endif
	virtual ~QTCPSock();

	/* Return bounding address of socket */
	virtual QNetAddr* GetBounds();

	/* Return remote peer information */
	virtual bool GetPeer(QNetAddr*);

	/* Open/close socket (connect/disconnect) */
	virtual bool Open();
	virtual void Close();

	/* Get/Set socket, return previous one */
#ifndef _WIN32
	virtual int SetSock(int nSock);
	virtual int GetSock();
#else
	virtual SOCKET SetSock(SOCKET nSock);
	virtual SOCKET GetSock();
#endif
	/* Is socket open? */
	virtual bool IsOpen();

	/* Generic send/receive */
	virtual bool GenRecv(void* pData, int nLen);
	virtual bool GenRecvTimed(void* pData, int nLen, int nTimeOut);
	virtual bool GenSend(void* pData, int nLen);
	virtual bool GenSendTimed(void* pData, int nLen, int nTimeOut);
	virtual bool GenSend(struct msghdr&, int nTotal);

	/* Timeout operations */
	virtual void SetTimeOut(int nTimeOut);
	virtual int GetTimeOut();
	virtual void SetTimed(bool bTimed);
	virtual bool GetTimed();

	/* Socket option setup */
	virtual void SetNodelay();
	virtual void SetCork(bool);
	virtual void SetHWKA();
	virtual bool SetOTTBuffers();
	virtual bool SetBuffers(int nSizeIn, int nSizeOut);

	/* Some ping to check socket is alive */
	virtual bool CheckConnect();

	/* Get available bytes in input buffer */
	virtual bool GetInputSize(int*);

	/* Get available bytes in output buffer */
	virtual bool GetOutputSize(int*);

	/* Chunk size setup */
	inline int GetSendSize() { return m_nSendChunk; }
	inline int GetRecvSize() { return m_nRecvChunk; }
	inline void SetSendSize(int nSize) { m_nSendChunk = nSize; }
	inline void SetRecvSize(int nSize) { m_nRecvChunk = nSize; }

	/* Send/receive typed functions */
	virtual bool SendByte(uint8_t);
	virtual bool SendWord(uint16_t);
	virtual bool SendDWord(uint32_t);
	virtual bool SendQWord(uint64_t);
	virtual bool SendString(const char*);
	virtual bool SendBool(bool);
	virtual bool SendFloat(float);
	virtual bool SendStr(std::string&);

	virtual bool RecvByte(uint8_t&);
	virtual bool RecvWord(uint16_t&);
	virtual bool RecvDWord(uint32_t&);
	virtual bool RecvQWord(uint64_t&);
	virtual bool RecvString(char*);
	virtual bool RecvBool(bool&);
	virtual bool RecvFloat(float&);
	virtual bool RecvStr(std::string&);

	/* Last error happened with socket */
	virtual int GetError();
protected:
	/* Error */
	void UpdateError();

#ifndef _WIN32
	int volatile 			m_sock;
#else
	SOCKET volatile 	m_sock;
#endif

	QNetAddr	m_bind;

	int				m_nError;
	bool			m_bTimed;
	int				m_nTimeOut;
	int				m_nSendChunk;
	int				m_nRecvChunk;
};

#endif
